/***********************************************************
*    SAM7SExx_DevMaster - Init of external memory NCSx     *
*                                                          *
*    generated by Frank Goetze  -   www.embedded-os.de     *
************************************************************
*                  based on MCK 48MHz                      *
***********************************************************/
#include "../inc/OS_Def_AT91SAM7xxx.h"

#define AT91S_NOR_BASE 	        0x10000000  // NCS0 Base address
#define AT91S_NOR_SIZE 	        0x00080000  // 512kB
#define AT91S_SDRAM_BASE        0x20000000  // NCS1 Base address
#define AT91S_SDRAM_SIZE        0x02000000  // 32MB
#define AT91S_SRAM_BASE         0x30000000  // NCS2 Base address
#define AT91S_SRAM_SIZE         0x00080000  // 512kB
#define AT91S_FRAM_BASE         0x60000000  // NCS5 Base address
#define AT91S_FRAM_SIZE         0x00080000  // 512kB
#define AT91S_EE_BASE           0x50000000  // NCS4 Base address
#define AT91S_EE_SIZE           0x00020000  // 128KB

void SAM7SE_MemInit(void)
{
    U32 i;
    /********************************************************
    *            init the external SMC A/D bus              *
    ********************************************************/
                                                           // Configure PIO controllers to periph mode
    AT91S_PIOB_BSR  = (AT91S_PB00_A0_NBS0 | AT91S_PB01_A1_NBS2 | AT91S_PB02_A2  | AT91S_PB03_A3  | AT91S_PB04_A4  | AT91S_PB05_A5  | AT91S_PB06_A6  | AT91S_PB07_A7  |
                       AT91S_PB08_A8      | AT91S_PB09_A9      | AT91S_PB10_A10 | AT91S_PB11_A11 | AT91S_PB12_A12 | AT91S_PB13_A13 | AT91S_PB14_A14 | AT91S_PB15_A15 |
                       AT91S_PB16_A16_BA0 | AT91S_PB17_A17_BA1 );
    AT91S_PIOB_PDR  = (AT91S_PB00_A0_NBS0 | AT91S_PB01_A1_NBS2 | AT91S_PB02_A2  | AT91S_PB03_A3  | AT91S_PB04_A4  | AT91S_PB05_A5  | AT91S_PB06_A6  | AT91S_PB07_A7  |
                       AT91S_PB08_A8      | AT91S_PB09_A9      | AT91S_PB10_A10 | AT91S_PB11_A11 | AT91S_PB12_A12 | AT91S_PB13_A13 | AT91S_PB14_A14 | AT91S_PB15_A15 |
                       AT91S_PB16_A16_BA0 | AT91S_PB17_A17_BA1 );
    AT91S_PIOC_ASR  = (AT91S_PC00_D0 | AT91S_PC01_D1 | AT91S_PC02_D2  | AT91S_PC03_D3  | AT91S_PC04_D4  | AT91S_PC05_D5  | AT91S_PC06_D6  | AT91S_PC07_D7  |
                       AT91S_PC08_D8 | AT91S_PC09_D9 | AT91S_PC10_D10 | AT91S_PC11_D11 | AT91S_PC12_D12 | AT91S_PC13_D13 | AT91S_PC14_D14 | AT91S_PC15_D15 |
                       AT91S_PC16_A18_NWAIT);
    AT91S_PIOC_BSR  = (AT91S_PC21_NWR0_NWE | AT91S_PC22_NRD);
    AT91S_PIOC_PDR  = (AT91S_PC00_D0 | AT91S_PC01_D1 | AT91S_PC02_D2  | AT91S_PC03_D3  | AT91S_PC04_D4  | AT91S_PC05_D5  | AT91S_PC06_D6  | AT91S_PC07_D7  |
                       AT91S_PC08_D8 | AT91S_PC09_D9 | AT91S_PC10_D10 | AT91S_PC11_D11 | AT91S_PC12_D12 | AT91S_PC13_D13 | AT91S_PC14_D14 | AT91S_PC15_D15 |
                       AT91S_PC16_A18_NWAIT | AT91S_PC21_NWR0_NWE | AT91S_PC22_NRD);
    /********************************************************
    *            init the NOR  interface on NCS0            *
    *                    ( AT49BV040B )                     *
    *  start:   0x 1000 0000       size:   0x 0008 0000     *
    ********************************************************/
    AT91S_PIOC_BSR  = AT91S_PC23_NCS0;                     // NCS0 config
    AT91S_PIOC_PDR  = AT91S_PC23_NCS0;
    AT91S_SMC_CSR->s[0] =                                  // Configure SMC Register
                      AT91S_SMC_CSR_NWS_2            |     // 1 wait states required by the NOR Flash device
                      AT91S_SMC_CSR_WSEN_EN          |     // NWS register enabled
                      AT91S_SMC_CSR_TDF_0            |     // 0 Data Float Time Cycles required by the NOR Flash device
                      AT91S_SMC_CSR_BAT_8BIT         |     // (one) 8-bit device connected over the bus
                      AT91S_SMC_CSR_DBW_8            |     // 8-bit Data Bus Width
                      AT91S_SMC_CSR_DRP_STD          |     // Standard Read protocol required by the NOR Flash device
                      AT91S_SMC_CSR_ACSS_STD         |     // Standard address to chip select
                      AT91S_SMC_CSR_RWSETUP_0        |     // 0 Read/Write Setup time required by the NOR Flash device
                      AT91S_SMC_CSR_RWHOLD_0         ;     // 0 Read/Write Hold time required by the NOR Flash device
    /********************************************************
    *           init the sdRAM interface on NCS1            *
    *                 ( MT48LC16M16A2-75 )                  *
    *  start:   0x 2000 0000       size:   0x 0200 0000     *
    ********************************************************/
    AT91S_EBI_CSA  |= AT91S_EBI_CS1A_SMC_SDRAMC;           // Chip Select is assigned to SMC SDRAM controller
    AT91S_PIOA_BSR  = (AT91S_PA23_NWR1_NBS1_NUB | AT91S_PA24_SDA10 | AT91S_PA25_SDCKE | AT91S_PA26_NCS1_SDCS |
                       AT91S_PA27_SDWE | AT91S_PA28_SDCAS | AT91S_PA29_SDRAS);
    AT91S_PIOA_PDR  = (AT91S_PA23_NWR1_NBS1_NUB | AT91S_PA24_SDA10 | AT91S_PA25_SDCKE | AT91S_PA26_NCS1_SDCS |
                       AT91S_PA27_SDWE | AT91S_PA28_SDCAS | AT91S_PA29_SDRAS);
    AT91S_SDRAMC_CR = AT91S_SDRAMC_CR_NC_9           |     // 9 Column bits
                      AT91S_SDRAMC_CR_NR_13          |     // 13 Row bits
                      AT91S_SDRAMC_CR_NB_4B          |     // 4 banks
                      AT91S_SDRAMC_CR_CAS_2          |     // Micron MT48LC16M16A2-75(100MHz) needs CAS 2
                      AT91S_SDRAMC_CR_TWR_2          |     // 2 Write Recovery cycles
                      AT91S_SDRAMC_CR_TRC_4          |     // 4 Row Delay cycles
                      AT91S_SDRAMC_CR_TRP_2          |     // 2 Row Precharge Delay cycles
                      AT91S_SDRAMC_CR_TRCD_2         |     // 2 Row Column Delay cycles
                      AT91S_SDRAMC_CR_TRAS_3         |     // 3 Active Precharge Delay cycles
                      AT91S_SDRAMC_CR_TXSR_4         ;     // 4 Exit Self Refresh Active Delay cycles
    for (i = 0; i < 0xFFFF; i++);                          // wait 200s here
    AT91S_SDRAMC_MR = (AT91S_SDRAMC_MR_DBW_16 | AT91S_SDRAMC_MR_MODE_NOP);
    *(volatile U32 *)0x20000000 = 0;                       // perform NOP
    AT91S_SDRAMC_MR = (AT91S_SDRAMC_MR_DBW_16 | AT91S_SDRAMC_MR_MODE_PRCH);
    *(volatile U32 *)0x20000000 = 0;                       // perform PRCHG ALL
    for (i = 0; i < 8; i++) {
        AT91S_SDRAMC_MR = (AT91S_SDRAMC_MR_DBW_16 | AT91S_SDRAMC_MR_MODE_RFSH);
        *(volatile U32 *)0x20000000 = 0;                   // perform 8 times CBR
    }
    AT91S_SDRAMC_MR = (AT91S_SDRAMC_MR_DBW_16 | AT91S_SDRAMC_MR_MODE_LMR);
    *(volatile U32 *)0x20000000 = 0;                       // perform LMR
    AT91S_SDRAMC_MR = (AT91S_SDRAMC_MR_DBW_16 | AT91S_SDRAMC_MR_MODE_NORMAL);
    *(volatile U32 *)0x20000000 = 0;                       // perform NORMAL mode
    AT91S_SDRAMC_TR = AT91S_SDRAMC_TR_VALUE;               // set refresh timer
    /********************************************************
    *            init the sRAM interface on NCS2            *
    *                   ( K6X4008T1F-70 )                   *
    *  start:   0x 3000 0000       size:   0x 0008 0000     *
    ********************************************************/
    AT91S_PIOA_BSR  = AT91S_PA20_NCS2;                     // NCS2 config
    AT91S_PIOA_PDR  = AT91S_PA20_NCS2;
    AT91S_SMC_CSR->s[2] =                                  // Configure SMC Register
                      AT91S_SMC_CSR_NWS_4            |     // 4 wait states required by the sRAM device
                      AT91S_SMC_CSR_WSEN_EN          |     // NWS register enabled
                      AT91S_SMC_CSR_TDF_1            |     // 1 Data Float Time Cycles required by the sRAM device
                      AT91S_SMC_CSR_BAT_8BIT         |     // (one) 8-bit device connected over the bus
                      AT91S_SMC_CSR_DBW_8            |     // 8-bit Data Bus Width
                      AT91S_SMC_CSR_DRP_STD          |     // Standard Read protocol required by the sRAM device
                      AT91S_SMC_CSR_ACSS_0           |     // 1 cycle address to chip select at begin and end
                      AT91S_SMC_CSR_RWSETUP_0        |     // 0 Read/Write Setup time required by the sRAM device
                      AT91S_SMC_CSR_RWHOLD_0         ;     // 0 Read/Write Hold time required by the sRAM device
    /********************************************************
    *            init the NAND interface on NCS3            *
    *            ( K9F1G08U0A / unknown device )            *
    *  start:   0x 4000 0000       size:   byte-serial      *
    ********************************************************/
    AT91S_EBI_CSA  |= AT91S_EBI_CS3A_SMC_NandFlash;        // Chip Select is assigned to SMC NAND controller
    AT91S_SMC_CSR->s[3] =                                  // Configure SMC Register
                      AT91S_SMC_CSR_NWS_2            |     // 2 wait states required by the NAND Flash device
                      AT91S_SMC_CSR_WSEN_EN          |     // NWS register enabled
                      AT91S_SMC_CSR_TDF_2            |     // 2 Data Float Time Cycles required by the NAND Flash device
                      AT91S_SMC_CSR_BAT_8BIT         |     // (one) 8-bit device connected over the bus
                      AT91S_SMC_CSR_DBW_8            |     // 8-bit Data Bus Width
                      AT91S_SMC_CSR_DRP_STD          |     // Standard Read protocol required by the NAND Flash device
                      AT91S_SMC_CSR_ACSS_STD         |     // Standard address to chip select
                      AT91S_SMC_CSR_RWSETUP_0        |     // 0 Read/Write Setup time required by the Nand Flash device
                      AT91S_SMC_CSR_RWHOLD_1         ;     // 1 Read/Write Hold time required by the ECC controller
    AT91S_PIOC_ASR  = (AT91S_PC19_A21_NANDALE | AT91S_PC20_A22_NANDCLE);
    AT91S_PIOC_BSR  = (AT91S_PC17_NANDOE | AT91S_PC18_NANDWE);
    AT91S_PIOC_PDR  = (AT91S_PC17_NANDOE | AT91S_PC18_NANDWE | AT91S_PC19_A21_NANDALE | AT91S_PC20_A22_NANDCLE);
    AT91S_PIOB_ODR  = AT91S_PB19;                          // Configure Ready/Busy PIO Input signal
    AT91S_PIOB_PER  = AT91S_PB18 | AT91S_PB19;
    AT91S_PIOB_SODR = AT91S_PB18;                          // Configure /CE PIO Output signal
    AT91S_PIOB_OER  = AT91S_PB18;                          // ... Disable Pull ups on A21/NANDALE and A22/NANDCLE
    AT91S_PIOC_PPUDR= (AT91S_PC19_A21_NANDALE | AT91S_PC20_A22_NANDCLE);
    AT91S_PMC_PCER  = (1 << AT91S_ID_PIOB);                // enable clock on PIO-B
    /********************************************************
    *            init the EE   interface on NCS4            *
    *                   ( AT28LV010-20 )                    *
    *  start:   0x 5000 0000       size:   0x 0002 0000     *
    ********************************************************/
    AT91S_PIOA_BSR  = AT91S_PA19_NCS4;                     // NCS4 config
    AT91S_PIOA_PDR  = AT91S_PA19_NCS4;
    AT91S_SMC_CSR->s[4] =                                  // Configure SMC Register
                      AT91S_SMC_CSR_NWS_9            |     // 9 wait states required by the EE device
                      AT91S_SMC_CSR_WSEN_EN          |     // NWS register enabled
                      AT91S_SMC_CSR_TDF_0            |     // 0 Data Float Time Cycles required by the EE device
                      AT91S_SMC_CSR_BAT_8BIT         |     // (one) 8-bit device connected over the bus
                      AT91S_SMC_CSR_DBW_8            |     // 8-bit Data Bus Width
                      AT91S_SMC_CSR_DRP_STD          |     // Standard Read protocol required by the EE device
                      AT91S_SMC_CSR_ACSS_STD         |     // Standard address to chip select
                      AT91S_SMC_CSR_RWSETUP_0        |     // 0 Read/Write Setup time required by the EE device
                      AT91S_SMC_CSR_RWHOLD_0         ;     // 0 Read/Write Hold time required by the EE device
    /********************************************************
    *            init the FRAM interface on NCS5            *
    *                     ( FM22L16-55 )                    *
    *  start:   0x 6000 0000       size:   0x 0008 0000     *
    ********************************************************/
    AT91S_PIOA_BSR  = AT91S_PA22_NCS5;                     // NCS5 config
    AT91S_PIOA_PDR  = AT91S_PA22_NCS5;
    AT91S_SMC_CSR->s[5] =                                  // Configure SMC Register
                      AT91S_SMC_CSR_NWS_4            |     // 4 wait states required by the fRAM device
                      AT91S_SMC_CSR_WSEN_EN          |     // NWS register enabled
                      AT91S_SMC_CSR_TDF_1            |     // 1 Data Float Time Cycles required by the fRAM device
                      AT91S_SMC_CSR_BAT_16BIT        |     // a 16-bit device connected over the bus
                      AT91S_SMC_CSR_DBW_16           |     // 16-bit Data Bus Width
                      AT91S_SMC_CSR_DRP_STD          |     // Standard Read protocol required by the fRAM device
                      AT91S_SMC_CSR_ACSS_1           |     // 1 cycle address to chip select at begin and end
                      AT91S_SMC_CSR_RWSETUP_0        |     // 0 Read/Write Setup time required by the fRAM device
                      AT91S_SMC_CSR_RWHOLD_0         ;     // 0 Read/Write Hold time required by the fRAM device
    /********************************************************
    *            init the ext. interface on NCS6            *
    *                  ( unknown device )                   *
    *  start:   0x 7000 0000       size:   max 256MB        *
    ********************************************************/
    AT91S_PIOA_BSR  = AT91S_PA21_NCS6;                     // NCS6 config
    AT91S_PIOA_PDR  = AT91S_PA21_NCS6;
    AT91S_SMC_CSR->s[6] =                                  // Configure SMC Register
                      AT91S_SMC_CSR_NWS_4            |     // 4 wait states required by the ext. device
                      AT91S_SMC_CSR_WSEN_EN          |     // NWS register enabled
                      AT91S_SMC_CSR_TDF_1            |     // 1 Data Float Time Cycles required by the ext. device
                      AT91S_SMC_CSR_BAT_16BIT        |     // a 16-bit device connected over the bus
                      AT91S_SMC_CSR_DBW_16           |     // 16-bit Data Bus Width
                      AT91S_SMC_CSR_DRP_STD          |     // Standard Read protocol required by the ext. device
                      AT91S_SMC_CSR_ACSS_1           |     // 1 cycle address to chip select at begin and end
                      AT91S_SMC_CSR_RWSETUP_0        |     // 0 Read/Write Setup time required by the ext. device
                      AT91S_SMC_CSR_RWHOLD_0         ;     // 0 Read/Write Hold time required by the ext. device
    AT91S_PIOB_ASR  = AT91S_PB20_IRQ0;
    AT91S_PIOB_PDR  = AT91S_PB20_IRQ0;                     // set to IRQ0 and enable pullup-resistor
    AT91S_PIOC_PPUDR= AT91S_PB20_IRQ0;
    /********************************************************
    *               init the user-LED on PA18               *
    *                    ( user-LED pin )                   *
    *                                                       *
    ********************************************************/
    AT91S_PIO_PER   = AT91S_PIO_PA18;                      // enable GPIO of user-LED
    AT91S_PIO_SODR  = AT91S_PIO_PA18;                      // set high (LED-off)
    AT91S_PIO_OER   = AT91S_PIO_PA18;                      // output enable
    AT91S_PMC_PCER  = (1 << AT91S_ID_PIO);                 // enable clock on PIO-A
}

/************************* END ***************************/


